]> Shamusworld >> Repos - rmac/blobdiff - token.h
Version bump for last patch; now at v1.13.4.
[rmac] / token.h
diff --git a/token.h b/token.h
index 4ea67028f5c5ffada4e0092b7da0fc4db0fce822..e20402bfd59db523f888ba41ad9a241c1e173f5d 100644 (file)
--- a/token.h
+++ b/token.h
@@ -1,7 +1,7 @@
 //
-// RMAC - Reboot's Macro Assembler for the Atari Jaguar Console System
+// RMAC - Reboot's Macro Assembler for all Atari computers
 // TOKEN.H - Token Handling
-// Copyright (C) 199x Landon Dyer, 2017 Reboot and Friends
+// Copyright (C) 199x Landon Dyer, 2011-2018 Reboot and Friends
 // RMAC derived from MADMAC v1.07 Written by Landon Dyer, 1986
 // Source utilised with the kind permission of Landon Dyer
 //
@@ -12,9 +12,9 @@
 #include "rmac.h"
 
 // Include Files and Macros
-#define SRC_IFILE       0                                      // Input source is IFILE
-#define SRC_IMACRO      1                                      // Input source is IMACRO
-#define SRC_IREPT       2                                      // Input source is IREPT
+#define SRC_IFILE       0                      // Input source is IFILE
+#define SRC_IMACRO      1                      // Input source is IMACRO
+#define SRC_IREPT       2                      // Input source is IREPT
 
 // Macros
 #define INOBJ           struct _inobj
 #define IFENT           struct _ifent
 
 // Tunable definitions
-#define LNSIZ           256                                    // Maximum size of a line of text
-#define TOKBUFSIZE      400                                    // Size of token-line buffer
-#define QUANTUM         4096L                          // # bytes to eat at a time from a file
-#define LNBUFSIZ        (QUANTUM*2)                    // Size of file's buffer
-#define KWSIZE          7                                      // Maximum size of keyword in kwtab.h
+#define LNSIZ           1024           // Maximum size of a line of text
+#define TOKBUFSIZE      400                    // Size of token-line buffer
+#define QUANTUM         4096L          // # bytes to eat at a time from a file
+#define LNBUFSIZ        (QUANTUM*2)    // Size of file's buffer
+#define KWSIZE          7                      // Maximum size of keyword in kwtab.h
 
 // (Normally) non-printable tokens
-#define COLON           ':'                                    // : (grumble: GNUmacs hates ':')
-#define CONST           'a'                                    // CONST <value>
-#define ACONST          'A'                                    // ACONST <value> <attrib>
-#define STRING          'b'                                    // STRING <address>
-#define SYMBOL          'c'                                    // SYMBOL <address>
-#define EOL             'e'                                    // End of line
-#define TKEOF           'f'                                    // End of file (or macro)
-#define DEQUALS         'g'                                    // ==
-#define SET             149                                    // Set
-#define REG             'R'                                    // Reg
-#define DCOLON          'h'                                    // ::
-#define GE              'i'                                    // >=
-#define LE              'j'                                    // <=
-#define NE              'k'                                    // <> or !=
-#define SHR             'l'                                    // >>
-#define SHL             'm'                                    // <<
-#define UNMINUS         'n'                                    // Unary '-'
-#define DOTB            'B'                                    // .b or .B or .s or .S
-#define DOTW            'W'                                    // .w or .W
-#define DOTL            'L'                                    // .l or .L
-#define DOTI            'I'                                    // .l or .L
-#define ENDEXPR         'E'                                    // End of expression
+#define COLON           ':'                    // : (grumble: GNUmacs hates ':')
+#define CONST           'a'                    // CONST <value>
+#define FCONST          'r'                    // Floating CONST <value>
+#define ACONST          'A'                    // ACONST <value> <attrib>
+#define STRING          'b'                    // STRING <address>
+#define STRINGA8        'S'                    // Atari 800 internal STRING <address>
+#define SYMBOL          'c'                    // SYMBOL <address>
+#define EOL             'e'                    // End of line
+#define TKEOF           'f'                    // End of file (or macro)
+#define DEQUALS         'g'                    // ==
+#define SET             0x95           // Set
+#define REG             'R'                    // Reg
+#define EQUREG          0x94           // equreg
+#define CCDEF           0xB7           // ccdef
+#define DCOLON          'h'                    // ::
+#define GE              'i'                    // >=
+#define LE              'j'                    // <=
+#define NE              'k'                    // <> or !=
+#define SHR             'l'                    // >>
+#define SHL             'm'                    // <<
+#define UNMINUS         'n'                    // Unary '-'
+#define DOTB            'B'                    // .b or .B or .s or .S
+#define DOTW            'W'                    // .w or .W
+#define DOTL            'L'                    // .l or .L
+#define DOTI            'I'                    // .l or .L
+#define DOTX            'X'                    // .x or .X
+#define DOTD            'D'                    // .d or .D
+#define DOTP            'P'                    // .p or .P
+#define DOTQ            'Q'                    // .q or .Q (essentially an alias for P)
+#define DOTS            'S'                    // .s or .S (FPU Single)
+#define ENDEXPR         'E'                    // End of expression
 
 // ^^ operators
-#define CR_DEFINED      'p'                                    // ^^defined - is symbol defined?
-#define CR_REFERENCED   'q'                                    // ^^referenced - was symbol referenced?
-#define CR_STREQ        'v'                                    // ^^streq - compare two strings
-#define CR_MACDEF       'w'                                    // ^^macdef - is macro defined?
-#define CR_TIME         'x'                                    // ^^time - DOS format time
-#define CR_DATE         'y'                                    // ^^date - DOS format date
-#define CR_ABSCOUNT     'z'                                    // ^^abscount - count the number of bytes defined in curent .abs section
+#define CR_DEFINED      'p'                    // ^^defined - is symbol defined?
+#define CR_REFERENCED   'q'                    // ^^referenced - was symbol referenced?
+#define CR_STREQ        'v'                    // ^^streq - compare two strings
+#define CR_MACDEF       'w'                    // ^^macdef - is macro defined?
+#define CR_TIME         'x'                    // ^^time - DOS format time
+#define CR_DATE         'y'                    // ^^date - DOS format date
+#define CR_ABSCOUNT     'z'                    // ^^abscount - count the number of bytes
+                                                                       // defined in curent .abs section
 
 // Character Attributes
-#define ILLEG           0                                      // Illegal character (unused)
-#define DIGIT           1                                      // 0-9
-#define HDIGIT          2                                      // A-F, a-f
-#define STSYM           4                                      // A-Z, a-z, _~.
-#define CTSYM           8                                      // A-Z, a-z, 0-9, _~$?
-#define SELF            16                                     // Single-character tokens: ( ) [ ] etc
-#define WHITE           32                                     // Whitespace (space, tab, etc.)
-#define MULTX           64                                     // Multiple-character tokens
-#define DOT             128                                    // [bwlsBWSL] for what follows a `.'
+#define ILLEG           0                      // Illegal character (unused)
+#define DIGIT           1                      // 0-9
+#define HDIGIT          2                      // A-F, a-f
+#define STSYM           4                      // A-Z, a-z, _~.
+#define CTSYM           8                      // A-Z, a-z, 0-9, _~$?
+#define SELF            16                     // Single-character tokens: ( ) [ ] etc
+#define WHITE           32                     // Whitespace (space, tab, etc.)
+#define MULTX           64                     // Multiple-character tokens
+#define DOT             128                    // [bwlsBWSL] for what follows a '.'
+
+// Macro to check for specific optimizations or override
+#define CHECK_OPTS(x)  (optim_flags[x] && !optimizeOff)
 
 // Conditional assembly structures
 IFENT {
@@ -81,7 +94,7 @@ IFENT {
        WORD if_state;                  // 0:enabled, 1:disabled
 };
 
// Pointer to IFILE or IMACRO
 IUNION {
        IFILE * ifile;
        IMACRO * imacro;
@@ -90,12 +103,12 @@ IUNION {
 
 // Ptr to IFILEs, IMACROs, and so on; back-ptr to previous input objects
 INOBJ {
-       WORD in_type;                   // 0=IFILE, 1=IMACRO ...
+       WORD in_type;                   // 0=IFILE, 1=IMACRO, 2=IREPT
        IFENT * in_ifent;               // Pointer to .if context on entry
        INOBJ * in_link;                // Pointer to previous INOBJ
        TOKEN * in_otok;                // Old `tok' value
        TOKEN * in_etok;                // Old `etok' value
-       IUNION inobj;                   // IFILE or IMACRO ...
+       IUNION inobj;                   // IFILE or IMACRO or IREPT
 };
 
 // Information about a file
@@ -119,21 +132,21 @@ TOKENSTREAM {
 // Information about a macro invocation
 IMACRO {
        IMACRO * im_link;               // Pointer to ancient IMACROs
-       struct LineList * im_nextln;    // Next line to include
+       LLIST * im_nextln;              // Next line to include
        WORD im_nargs;                  // # of arguments supplied on invocation
        WORD im_siz;                    // Size suffix supplied on invocation
        LONG im_olduniq;                // Old value of 'macuniq'
        SYM * im_macro;                 // Pointer to macro we're in
        char im_lnbuf[LNSIZ];   // Line buffer
-       uint32_t argBase;               // Base in argPtrs[] for current macro
        TOKENSTREAM argument[20];       // Assume no more than 20 arguments in an invocation
 };
 
 // Information about a .rept invocation
 IREPT {
-       LONG * ir_firstln;              // Pointer to first line
-       LONG * ir_nextln;               // Pointer to next line
-       VALUE ir_count;                 // Repeat count (decrements)
+       LLIST * ir_firstln;             // Pointer to first line
+       LLIST * ir_nextln;              // Pointer to next line
+       uint32_t ir_count;              // Repeat count (decrements)
+       uint32_t lineno;                // Repeat line number (Convert this to global instead of putting it here?)
 };
 
 // Exported variables
@@ -148,6 +161,7 @@ extern char tolowertab[];
 extern INOBJ * cur_inobj;
 extern int mjump_align;
 extern char * string[];
+extern int optimizeOff;
 
 // Exported functions
 int include(int, char *);
@@ -157,6 +171,7 @@ int TokenizeLine(void);
 int fpop(void);
 int d_goto(WORD);
 INOBJ * a_inobj(int);
+void DumpToken(TOKEN);
 void DumpTokenBuffer(void);
 
 #endif // __TOKEN_H__
@@ -81,7 +94,7 @@ IFENT {
 };
 
-// Pointer to IFILE or IMACRO
+// Pointer to IFILE or IMACRO or IREPT
 IUNION {
        IFILE * ifile;